/*--------------------------------------------------------------
// file: bigG.c - compute euclidian distance matrix
//   D = bigG(A,B,sigmaSqr)
// INPUTS:
//   A - mxd matrix (m=number of points, d=number of dimensions)
//   B - kxd matrix (k=number of points, d=number of dimensions)
//   sigmaSqr - Gaussian variance 
// OUTPUTS:
//   D - mxk matrix of Gaussian evaluations i.e. element (i,j) of D is
//       G( A(i,:) - B(j,:), sigmaSqr ) = 
//
//       exp( -(1/2) ||A(i,:)-B(j,:)||^2 / sigmaSqr ) / (2 pi sigmaSqr)^(d/2)
//
// Checked:
//--------------------------------------------------------------*/
#include <math.h>
#include "mex.h"

/*--------------------------------------------------------------
// function: mexFunction - Entry point from Matlab environment
// INPUTS:
//   nlhs - number of left hand side arguments (outputs)
//   plhs[] - pointer to table where created matrix pointers are
//            to be placed
//   nrhs - number of right hand side arguments (inputs)
//   prhs[] - pointer to table of input matrices
//--------------------------------------------------------------*/
void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )
{
  int M,N,K; /* Sizes */
  int m,n,k; /* Loop variables */
  double *pA, *pB, *pD, *pSS, denom;

  if (nrhs < 3)
    mexErrMsgTxt("Not enough input arguments.");
  if (nrhs > 3)
    mexErrMsgTxt("Too many input arguments.");
  if (!mxIsDouble(prhs[0]) || !mxIsDouble(prhs[1]) || !mxIsDouble(prhs[2]))
    mexErrMsgTxt("Arguments must be type double.");
  if (mxGetN(prhs[0]) != mxGetN(prhs[1]))
    mexErrMsgTxt("matrix dimensions (number of columns) must agree.");
  if (mxIsComplex(prhs[0]) || mxIsComplex(prhs[1]))
    mexWarnMsgTxt("Complex parts ignored.");

  /* bigG(MxK,NxK) => MxN */
  M = mxGetM(prhs[0]); /* Number of rows in output */
  N = mxGetM(prhs[1]); /* Number of columns in output */
  K = mxGetN(prhs[0]); /* Number of dimensions */

  plhs[0] = mxCreateDoubleMatrix(M,N,mxREAL);

  pA  = mxGetPr(prhs[0]);
  pB  = mxGetPr(prhs[1]);
  pSS = mxGetPr(prhs[2]); 
  pD  = mxGetPr(plhs[0]);

  /* Now the distance computation */
  pA = mxGetPr(prhs[0]);
  pB = mxGetPr(prhs[1]);
  denom = pow( 2*M_PI*pSS[0], K/2 ); 
  for (m=0; m<M; m++)
  {
    for (n=0; n<N; n++)
    {
      double sum = 0.0, diff;
      for (k=0; k<K; k++)
      {
        diff = pA[m+M*k] - pB[n+N*k];
        sum += diff*diff;
      }
      pD[m+M*n] = exp( -0.5*sum/pSS[0] )/denom;
    }
  } 
} /* end mexFunction() */
