function [ DVCY2, DVC2Y2, DVCY ] = computeInformationForces(Ys,k,Jp,sgma)
%
% The output is the derivatives of the information potential for
% every data point in each class instance.  Each matrix is of size
% [ d x N ], where d is the dimension of the output space and N is 
% the number of points 
% 
% This implements equations 38,39, and 40
% 
% Note: this outputs derivatives of the mutual information w.r.t. y_{c,i}
% ordered with class 1 first, class 2 next, ... up to Nc.
% 
% Warning: this code may not work if there are more classes than are 
% represented in the vector k, i.e. if Jp = [ 2 0 ] with k = [ 1 1 ].
%
% Written by:
% -- 
% John L. Weatherwax                2005-10-30
% 
% email: wax@alum.mit.edu
% 
% Please send comments and especially bug reports to the
% above email address.
% 
%-----

N = size(Ys,1); 
Nc = length(unique(k)); 

%
% Each type of derivatives for each point of each class:
%

DVCY2 = [];
for c=1:Nc
  Yc = Ys(c==k,:);
  DVCY2 = [ DVCY2, dVsubcy2( Yc, sgma, N ) ]; 
end

DVC2Y2 = dVsubc2y2( Ys, sgma, Jp, N );

DVCY = []; 
for c=1:Nc
  DVCY = [ DVCY, dVsubcy( Ys, k, sgma, c, Jp ) ]; 
end

%--
% Auxilliary subroutines below here:
%--

function [ res ] = dVsubcy2( Ys, sgma, N )
%
% Implements Eq. 38
% --checked (2005-11-12)
%
%-----

d = size( Ys, 2 ); 
G = bigG( Ys, Ys, 2*sgma^2 );
Gr = repmat( G, [ 1, 1, d ] ); 
YD = vdist( Ys, Ys );
tmpRes = sum( Gr .* YD, 1 );
denom = (N*sgma)^2; 
res = ( ( squeeze( tmpRes ) ).' )/denom;
if( size(res,1) ~= d )
  res=res.'; 
end


function [ res ] = dVsubc2y2( Ys, sgma, Jp, N )
%
% Implements Eq. 39.
% --checked (2005-11-12)
% 
%-----
  
d = size( Ys, 2 ); 
G = bigG( Ys, Ys, 2*sgma^2 );
Gr = repmat( G, [ 1, 1, d ] ); 
YD = vdist( Ys, Ys );
tmpRes = sum( Gr .* YD, 1 );
numer = sum( Jp.^2 )/(N^2); 
denom = (N*sgma)^2; 
res = numer*( ( squeeze( tmpRes ) ).' )/denom;



function [ res ] = dVsubcy( Ys, k, sgma, c, Jp )
%
% Implements Eq. 40
% --checked (2005-11-12)
% 
% Ys represent "all" patterns (not only the class specific ones)
% k are the corresponding class labels
% c is the specific class this derivative is computed for
%
%-----

N = size( Ys, 1 ); 
d = size( Ys, 2 ); 
Nc = length(unique(k)); 
Yc = Ys( k==c, : ); 
%res = zeros(d,Jp(c)); 
for p=1:Nc
  Yp = Ys( k==p, : ); 
  G = bigG( Yp, Yc, 2*sgma^2 );  
  Gr = repmat( G, [ 1, 1, d ] ); 
  YD = vdist( Yp, Yc );
  ptRes = sum( Gr .* YD, 1 );
  if( p==1 ) % initialization
    res =       ( (Jp(p)+Jp(c))/(2*N) ) * ( squeeze( ptRes ).' );
  else
    res = res + ( (Jp(p)+Jp(c))/(2*N) ) * ( squeeze( ptRes ).' );
  end
end
if( size(res,1) ~= d )
  res=res.'; 
end
denom = (N*sgma)^2; 
res = res/denom;
